import java.util.*;


/* Our solution is based on a "reduction" the Nim problem. 
 *
 * Nim is also a two-player game, and starts off a number of heaps of "objects" (we will distinguish this 
 * from "stones" for reasons discussed below). Players take turns choosing a heap and removing 1 or more
 * objects from that heap. The game continues till there are no objects left, and the player making the 
 * last move wins.
 *
 * Nim has a really nice solution -- which almost seems like magic. It is based on taking an XOR of 
 * the binary representations of the numbers of objects in the heaps. Read the Wikipedia article for 
 * details on that.
 *
 * It turns out: our game can be reduced to Nim using a clever reduction. 
 *
 * We will number the nodes (not in a unique fashion) in our Acyclic Directed graph as follows. 
 *
 * We say that a node can "see" a number Y, if it has an directed edge TO a node numbered Y.
 *
 * Our numbering has a simple rule: a node is numbered X if it can see numbers 0 through X-1,
 * but not X.
 *
 * So all sinks are numbered 0. A node that has an outgoing edge to 0 but no other outgoing 
 * edge is numbered 1, and so on. 
 *
 * Finally, the "reduction":  for every stone in our game, we create a separate heap in Nim containing
 * exactly the same number of objects as the "number" of the node at which the stone is located. 
 *
 * So, for the first example: the "numbers" associated with the four nodes from right to left are: 
 * 0, 1, 0, 1, and hence we will create a NIM instance with just one heap containing 1 object.
 *
 * For the second example: we create a NIM instance with 3 heaps, containing: 3, 1, 2 objects
 * (0th node has number 3, 2nd node has number 1, and 4th node has number 2).  
 *
 * Once this reduction is established, the problem is straightforward. The numbering can be
 * done in a recursive manner, and then we just need to take an XOR. */

public class Stones {
    static boolean mark[];
    private static int find(int v, ArrayList graph[], int grundynum[], boolean mark[]){
        if(mark[v])
            return grundynum[v];

        mark[v] = true;

        TreeSet<Integer> st = new TreeSet<Integer>();

        /* Retrieve all the "numbers" that I can "see". */
        for(int i= 0; i < graph[v].size(); i++)
            st.add(find(((Integer) graph[v].get(i)).intValue(),graph, grundynum, mark));

        /* Find the smallest number that I cannot "see". */
        for(int i = 0; i < 1001; i++)
            if(!st.contains(i)){
                grundynum[v] = i;
                break;
            }
        return grundynum[v];
    }

    /* Find the "numbers" for all the nodes". */
    public static void findgrund(int numNodes, ArrayList graph[], int grundynum[]){
        mark = new boolean[numNodes];

        for(int i = 0; i < numNodes; i++)
            grundynum[i] = find(i, graph, grundynum, mark);
    }

    private static boolean solveStones(int numNodes, ArrayList[] graph, int[] numStones) {
        boolean firstWins = false;

        /* ------------------- INSERT CODE HERE ---------------------*/
        int grundynum[] = new int[numNodes];

        findgrund(numNodes, graph, grundynum);

        int res = 0;

        for(int i = 0; i < numNodes; i++)
            if(numStones[i] % 2 == 1) // two stones at a single node will cancel when then XOR
                res ^= grundynum[i];

        firstWins = (res != 0);

        /* -------------------- END OF INSERTION --------------------*/
        return firstWins;
    }

    public static void main(String args[]) {
        int numNodes, numEdges;
        ArrayList[] graph;
        int[] numStones;
        
        Scanner sc = new Scanner(System.in);

        while(sc.hasNext()){

            numNodes = sc.nextInt();

            numEdges = sc.nextInt();

            if (numNodes == 0)
                break;

            graph = new ArrayList[numNodes];

            numStones = new int[numNodes];

            /* Create nodes. */
            for(int i = 0; i < numNodes; i++)
                graph[i] = new ArrayList<Integer>();

            /* Read the edge information and create edges. */
            for(int i = 0; i < numEdges; i++)
                graph[sc.nextInt()].add(sc.nextInt());

            /* Read the number of stones on each node. */
            for(int i =0; i < numNodes; i++)
                numStones[i] = sc.nextInt();

            if(solveStones(numNodes, graph, numStones))
                System.out.println("First");
            else
                System.out.println("Second");
        }
    }
}
