//----------------------------------------------------------------------
// Best viewed with tabs every 4 columns
//----------------------------------------------------------------------
import java.util.*;

/**
 * Determines whether a string can be generated by a given set of
 * production rules.
 *
 * We are given a collection of production rules of the form "x --> yz",
 * meaning that the character x can be replaced with the string "yz".
 * Given such a collection of rules, and a pair consisting of an input
 * symbol and an output string, and the question is there a way to
 * generate the output string through a series of substitutions to the
 * input symbol. If so, a derivation of the sequence is to be output.
 */
public class DNAManipulator {

	/** 
	 *  Main program
	 *
	 *	Here is the input format:
	 *			<number of cases>
	 *			(Repeat the following for each case)
	 *				<lhs>  <rhs>	(the production "lhs --> rhs"
	 *				<lhs>  <rhs>
	 *				...
	 *				$				(marks end of productions)
	 *				<start>			(start symbol)
	 *				<target>		(target string)
	 *
	 *	The programmer provides computeDerivation, which determines
	 *	whether it is possible to derive the target from the start
	 *	symbol. (See below.)
	 */
	public static void main(String[] args) throws Exception {
											// read input
		Scanner scanner = new Scanner(System.in);
		int nTests = scanner.nextInt();		// number of test cases

		for (int t = 0; t < nTests; t++) {
			System.out.println("Test case: " + t);
											// productions "lhs --> rhs"
			ArrayList<Character> lhs = new ArrayList<Character>();
			ArrayList<String> rhs = new ArrayList<String>();
											// input next lhs
			Character l = nextCharacter(scanner);
			while (l.charValue() != '$') {	// not yet finished
				lhs.add(l);					// add new lhs
				String r = scanner.next();	// next rhs
				rhs.add(r);					// add new rhs
				l = nextCharacter(scanner);	// input next lhs
			}
			summarizeProductions(lhs, rhs);	// summarize productions
											// start symbol
			Character start = nextCharacter(scanner);
			while (start.charValue() != '#') {	// not yet done
				String target = scanner.next();	// target string
											// compute derivation
				ArrayList<Integer> derivation =
					computeDerivation(lhs, rhs, start, target);
				checkAndSummarizeResult(lhs, rhs, start, target, derivation);
											// get next start
				start = nextCharacter(scanner);
			}
			System.out.println();
		}
	}

	/** 
	 *  Utility that reads a single character string
	 */
	private static Character nextCharacter(Scanner scanner) {
		String x = scanner.next();
		if (x.length() != 1) {
			System.out.println("Error: Expecting a single character");
			System.exit(0);
		}
		return new Character(x.charAt(0));
	}
	
	/** 
	 *  Summarize input
	 */
	private static void summarizeProductions(
			ArrayList<Character> lhs,		// left-hand-sides
			ArrayList<String> rhs)			// corresponding right-hand-sides
	{
		System.out.println("  Productions: (lhs --> rhs)");
		int nr = lhs.size();				// number of rules
		if (nr != rhs.size()) {
			System.out.println("Error: lhs and rhs must have equal length");
			System.exit(0);
		}
		for (int i = 0; i < nr; i++) {
			System.out.println("    [" + i + "]: " + lhs.get(i) + " --> " + rhs.get(i));
			if (rhs.get(i).length() != 2) {
				System.out.println("Error: rhs must be two characters");
				System.exit(0);
			}
		}
		System.out.println();
	}
	
	/** 
	 *  Summarize derivation and verify its correctness
	 */
	private static void checkAndSummarizeResult(
			ArrayList<Character> lhs,		// left-hand-sides
			ArrayList<String> rhs,			// corresponding right-hand-sides
			Character start,				// start symbol
			String target,					// target string
			ArrayList<Integer> derivation)	// derivation
	{
		int n = target.length();			// target size
		int nr = lhs.size();				// number of rules
		int m = derivation.size()/2;		// length of derivation
		System.out.print("  Derive \"" + target + "\" from \"" + start + "\"? ");
		if (m == 0) {						// empty derivation?
			if (target.length() == 1 &&		// okay, if target == start
				start.charValue() == target.charAt(0)) {
				System.out.println("YES! (trivially)");
			}
			else {							// otherwise, no derivation
				System.out.println("NO!");
			}
		}
		else {								// nontrivial derivation
			System.out.println("YES!");

			System.out.print("    Derivation: ");
			for (int i = 0; i < m; i++) {	// print it
				System.out.print("(" + derivation.get(2*i) + "," + derivation.get(2*i+1) + ") ");
			}
			System.out.println();

			StringBuffer current = new StringBuffer();
			current.append(start);			// current = start symbol
			for (int i = 0; i < m; i++) {
				int r = derivation.get(2*i);	// get rule
				int p = derivation.get(2*i + 1);// get application index
				if (r < 0 || r >= nr) {		// check rule index for validity
					System.out.println("Error - rule index " + r + " is out of bounds");
					System.exit(0);
				}
											// rule: s --> x y
				char s = lhs.get(r).charValue();
				String thisRhs = rhs.get(r);
				char x = thisRhs.charAt(0);
				char y = thisRhs.charAt(1);
											// check application index validity
				if (p < 0 || p >= current.length()) {
					System.out.println("Error - application index " + p + " is out of bounds");
					System.exit(0);
				}
											// check rule application validity
				if (s != current.charAt(p)) {
					System.out.println("Error - attempt to apply rule for " + s + " but character is " + current.charAt(p));
					System.exit(0);
				}
											// make current substitution
				current.replace(p, p+1, rhs.get(r));
				System.out.print("      Apply[" + r + "] at " + p + ": ");
				if (i == 0) System.out.print("S");
				else System.out.print(" ");
				System.out.println( " ==> " + current);
			}
			if (!current.toString().equals(target)) {
				System.out.println("Error - derivation does not yield target");
				System.exit(0);
			}
		}
	}
	 
	/** 
	 *  computeDerivation
	 *
	 *	Given a list of productions, a start symbol, and a target
	 *	string, this function determines whether it is possible to
	 *	derive the target from the start symbol using the productions.
	 *	It returns an ArrayList of Integers, of length 2*m, where m is
	 *	the number of steps in the derivation. If no derivation is
	 *	possible, or if start == target, this ArrayList is empty.
	 *	Otherwise, its entries come in pairs, where the first of each
	 *	pair indicates which production rule is to be applied, and the
	 *	second indicates the index of the current string at which to
	 *	apply the rule. (All indexing starts at 0.)
	 */

	private static ArrayList<Integer> computeDerivation(
			ArrayList<Character> lhs,		// left-hand-sides
			ArrayList<String> rhs,			// corresponding right-hand-sides
			Character start,				// start symbol
			String target)					// target string
	{
		// --- Add you code here ---
        return new ArrayList<Integer>();
	}

}
